/* Copyright (c) 2003 - Charles Schmidt <cschmidt2@emich.edu> */

#include "ishzilla-prompt-service.h"
#include "mozilla.h"

#include <nsCOMPtr.h>
#include <nsString.h>
#include <nsReadableUtils.h>
#include <nsIServiceManager.h>
#include <nsIDOMWindow.h>
#include <nsIWindowWatcher.h>
#include <nsIEmbeddingSiteWindow.h>
#include <nsIWebBrowserChrome.h>

#include <gtk/gtk.h>

static GtkWidget * gtk_widget_from_dom_window(nsIDOMWindow *window);
static void gtk_window_set_title_from_prunichar(GtkWindow *window,const PRUnichar *title);
static void gtk_button_set_text_from_prunichar(GtkButton *button,const PRUnichar *text);
static void gtk_label_set_text_from_prunichar(GtkLabel *label,const PRUnichar *text);
static void gtk_entry_set_text_from_prunichar(GtkEntry *entry,const PRUnichar *text);
static PRUnichar * gtk_entry_get_text_as_prunichar(GtkEntry *entry);


NS_IMPL_ISUPPORTS1(IshzillaPromptService,nsIPromptService);


IshzillaPromptService::IshzillaPromptService()
{
	NS_INIT_ISUPPORTS();
}

IshzillaPromptService::~IshzillaPromptService()
{

}

NS_IMETHODIMP IshzillaPromptService::Alert(nsIDOMWindow *parent,const PRUnichar *title,const PRUnichar *text)
{
	GtkWidget *dialog;
	GtkWidget *gtk_parent = gtk_widget_from_dom_window(parent);
	const nsACString &msg = NS_ConvertUCS2toUTF8(text);

	dialog = gtk_message_dialog_new(GTK_WINDOW(gtk_parent),GTK_DIALOG_MODAL,GTK_MESSAGE_WARNING,GTK_BUTTONS_OK,"%s",PromiseFlatCString(msg).get());

	gtk_window_set_title_from_prunichar(GTK_WINDOW(dialog),title);

	gtk_dialog_run(GTK_DIALOG(dialog));
	gtk_widget_destroy(dialog);

	return NS_OK;
}
	
NS_IMETHODIMP IshzillaPromptService::AlertCheck(nsIDOMWindow *parent,const PRUnichar *title,const PRUnichar *text,const PRUnichar *message,PRBool *value)
{
	GtkWidget *dialog;
	GtkWidget *gtk_parent = gtk_widget_from_dom_window(parent);
	GtkWidget *check_button;
	const nsACString &msg = NS_ConvertUCS2toUTF8(text);

	dialog = gtk_message_dialog_new(GTK_WINDOW(gtk_parent),GTK_DIALOG_MODAL,GTK_MESSAGE_WARNING,GTK_BUTTONS_OK,"%s",PromiseFlatCString(msg).get());

	if (value) {
		check_button = gtk_check_button_new_with_label("");
		gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox),check_button,FALSE,FALSE,5);
		gtk_button_set_text_from_prunichar(GTK_BUTTON(check_button),message);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button),*value);
		gtk_widget_show(check_button);
	}

	gtk_window_set_title_from_prunichar(GTK_WINDOW(dialog),title);

	gtk_dialog_run (GTK_DIALOG (dialog));

	if (value) {
		*value = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button));
	}

	gtk_widget_destroy (dialog);
	
	return NS_OK;
}

NS_IMETHODIMP IshzillaPromptService::Confirm(nsIDOMWindow *parent,const PRUnichar *title,const PRUnichar *text,PRBool *ret)
{
	GtkWidget *dialog;
	GtkWidget *gtk_parent = gtk_widget_from_dom_window(parent);
	const nsACString &msg = NS_ConvertUCS2toUTF8(text);
	gint r;

	dialog = gtk_message_dialog_new(GTK_WINDOW(gtk_parent),GTK_DIALOG_MODAL,GTK_MESSAGE_QUESTION,GTK_BUTTONS_YES_NO,"%s",PromiseFlatCString(msg).get());

	gtk_window_set_title_from_prunichar(GTK_WINDOW(dialog),title);

	r = gtk_dialog_run(GTK_DIALOG(dialog));

	*ret = (r == GTK_RESPONSE_YES);

	gtk_widget_destroy(dialog);

	return NS_OK;
}

NS_IMETHODIMP IshzillaPromptService::ConfirmCheck(nsIDOMWindow *parent,const PRUnichar *title,const PRUnichar *text,const PRUnichar *message,PRBool *value,PRBool *ret)
{
	GtkWidget *dialog;
	GtkWidget *gtk_parent = gtk_widget_from_dom_window(parent);
	GtkWidget *check_button;
	const nsACString &msg = NS_ConvertUCS2toUTF8 (text);
	gint r;

	dialog = gtk_message_dialog_new(GTK_WINDOW(gtk_parent),GTK_DIALOG_MODAL,GTK_MESSAGE_QUESTION,GTK_BUTTONS_YES_NO,"%s",PromiseFlatCString(msg).get());

	if (value) {
		check_button = gtk_check_button_new_with_label("");
		gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox),check_button,FALSE,FALSE,5);
		gtk_button_set_text_from_prunichar(GTK_BUTTON(check_button),message);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button),*value);
		gtk_widget_show(check_button);
	}

	gtk_window_set_title_from_prunichar(GTK_WINDOW(dialog),title);

	r = gtk_dialog_run (GTK_DIALOG (dialog));
	*ret = (r == GTK_RESPONSE_YES) ? PR_TRUE : PR_FALSE;

	if (value) {
		*value = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button));
	}

	gtk_widget_destroy (dialog);
	
	return NS_OK;
}

NS_IMETHODIMP IshzillaPromptService::add_button(GtkWidget *dialog,char type,const PRUnichar *text,gint id)
{
	const gchar *button;
	const nsACString &msg = NS_ConvertUCS2toUTF8 (text);

	switch (type) {
		case BUTTON_TITLE_OK:
			button = GTK_STOCK_OK;
			break;
		case BUTTON_TITLE_CANCEL:
			button = GTK_STOCK_CANCEL;
			break;
		case BUTTON_TITLE_YES:
			button = GTK_STOCK_YES;
			break;
		case BUTTON_TITLE_NO:
			button = GTK_STOCK_NO;
			break;
		case BUTTON_TITLE_SAVE:
			button = "Save";
			break;
		case BUTTON_TITLE_REVERT:
			button = "Revert";
			break;
		case BUTTON_TITLE_DONT_SAVE:
			button = "Don't save";
			break;
		case BUTTON_TITLE_IS_STRING:
			button = NULL;
			break;
		default:
			return NS_ERROR_FAILURE;
	}

	gtk_dialog_add_button(GTK_DIALOG(dialog),button ? button : PromiseFlatCString(msg).get(),id);

	return NS_OK;
}


NS_IMETHODIMP IshzillaPromptService::ConfirmEx(nsIDOMWindow *parent,const PRUnichar *title,const PRUnichar *text,PRUint32 button_flags,const PRUnichar *button0_text,const PRUnichar *button1_text,const PRUnichar *button2_text,const PRUnichar *message,PRBool *value,PRInt32 *button_pressed)
{
	GtkWidget *dialog;
	GtkWidget *gtk_parent = gtk_widget_from_dom_window(parent);
	GtkWidget *check_button;
	const nsACString &msg = NS_ConvertUCS2toUTF8 (text);
	gint r;

	dialog = gtk_message_dialog_new(GTK_WINDOW(gtk_parent),GTK_DIALOG_MODAL,GTK_MESSAGE_QUESTION,GTK_BUTTONS_NONE,"%s",PromiseFlatCString(msg).get());

	if (value) {
		check_button = gtk_check_button_new_with_label("");
		gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox),check_button,FALSE,FALSE,5);
		gtk_button_set_text_from_prunichar(GTK_BUTTON(check_button),message);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button),*value);
		gtk_widget_show(check_button);
	}

	add_button(dialog,(button_flags >> 16) & 0xFF,button2_text,2);
	add_button(dialog,(button_flags >> 8) & 0xFF,button1_text,1);
	add_button(dialog,button_flags & 0xFF,button0_text,0);

	gtk_dialog_set_default_response (GTK_DIALOG(dialog), 0);
	r = gtk_dialog_run (GTK_DIALOG (dialog));

	*button_pressed = r;

	if (value) {
		*value = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button));
	}

	gtk_widget_destroy (dialog);

	return NS_OK;
}



NS_IMETHODIMP IshzillaPromptService::Prompt(nsIDOMWindow *parent,const PRUnichar *title,const PRUnichar *text,PRUnichar **text_value,const PRUnichar *message,PRBool *value,PRBool *ret) 
{
	GtkWidget *dialog;
	GtkWidget *gtk_parent = gtk_widget_from_dom_window(parent);
	GtkWidget *entry;
	GtkWidget *check_button;
	const nsACString &msg = NS_ConvertUCS2toUTF8 (text);
	gint r;

	dialog = gtk_message_dialog_new(GTK_WINDOW(gtk_parent),GTK_DIALOG_MODAL,GTK_MESSAGE_QUESTION,GTK_BUTTONS_OK_CANCEL,"%s",PromiseFlatCString(msg).get());

	entry = gtk_entry_new();
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox),entry,FALSE,FALSE,5);
	gtk_entry_set_text_from_prunichar(GTK_ENTRY(entry),*text_value);
	gtk_widget_show(entry);

	if (value) {
		check_button = gtk_check_button_new_with_label("");
		gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox),check_button,FALSE,FALSE,5);
		gtk_button_set_text_from_prunichar(GTK_BUTTON(check_button),message);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button),*value);
		gtk_widget_show(check_button);
	}

	gtk_window_set_title_from_prunichar(GTK_WINDOW(dialog),title);

	r = gtk_dialog_run (GTK_DIALOG (dialog));
	*ret = (r == GTK_RESPONSE_OK) ? PR_TRUE : PR_FALSE;

	*text_value = gtk_entry_get_text_as_prunichar(GTK_ENTRY(entry));
	if (value) {
		*value = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button));
	}

	gtk_widget_destroy (dialog);
	
	return NS_OK;
}


NS_IMETHODIMP IshzillaPromptService::PromptUsernameAndPassword(nsIDOMWindow *parent,const PRUnichar *title,const PRUnichar *text,PRUnichar **username,PRUnichar **password,const PRUnichar *message,PRBool *value,PRBool *ret) 
{
	GtkWidget *dialog;
	GtkWidget *gtk_parent = gtk_widget_from_dom_window(parent);
	GtkSizeGroup *size_group;
	GtkWidget *table;
	GtkWidget *label;
	GtkWidget *username_entry;
	GtkWidget *password_entry;
	GtkWidget *check_button;
	const nsACString &msg = NS_ConvertUCS2toUTF8 (text);
	gint r;

	dialog = gtk_message_dialog_new(GTK_WINDOW(gtk_parent),GTK_DIALOG_MODAL,GTK_MESSAGE_QUESTION,GTK_BUTTONS_OK_CANCEL,"%s",PromiseFlatCString(msg).get());

	size_group = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);

	table = gtk_table_new(2,2,FALSE);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox),table,FALSE,FALSE,5);

	label = gtk_label_new("Username:");
	gtk_table_attach(GTK_TABLE(table),label,0,1,0,1,(GtkAttachOptions)(GTK_EXPAND | GTK_FILL),(GtkAttachOptions)0,0,0);

	username_entry = gtk_entry_new();
	gtk_size_group_add_widget(size_group,username_entry);
	gtk_table_attach(GTK_TABLE(table),username_entry,1,2,0,1,(GtkAttachOptions)0,(GtkAttachOptions)0,0,0);
	gtk_entry_set_text_from_prunichar(GTK_ENTRY(username_entry),*username);

	label = gtk_label_new("Password:");
	gtk_table_attach(GTK_TABLE(table),label,0,1,1,2,(GtkAttachOptions)(GTK_EXPAND | GTK_FILL),(GtkAttachOptions)0,0,0);

	password_entry = gtk_entry_new();
	gtk_entry_set_visibility(GTK_ENTRY(password_entry),FALSE);
	gtk_size_group_add_widget(size_group,password_entry);
	gtk_table_attach(GTK_TABLE(table),password_entry,1,2,1,2,(GtkAttachOptions)0,(GtkAttachOptions)0,0,0);
	gtk_entry_set_text_from_prunichar(GTK_ENTRY(password_entry),*password);

	gtk_widget_show_all(table);

	if (value) {
		check_button = gtk_check_button_new_with_label("");
		gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox),check_button,FALSE,FALSE,5);
		gtk_button_set_text_from_prunichar(GTK_BUTTON(check_button),message);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button),*value);
		gtk_widget_show(check_button);
	}

	gtk_window_set_title_from_prunichar(GTK_WINDOW(dialog),title);

	r = gtk_dialog_run (GTK_DIALOG (dialog));

	*ret = (r == GTK_RESPONSE_OK) ? PR_TRUE : PR_FALSE;

	*username = gtk_entry_get_text_as_prunichar(GTK_ENTRY(username_entry));
	*password = gtk_entry_get_text_as_prunichar(GTK_ENTRY(password_entry));
	if (value) {
		*value = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button));
	}

	gtk_widget_destroy (dialog);
	
	return NS_OK;

}


NS_IMETHODIMP IshzillaPromptService::PromptPassword(nsIDOMWindow *parent,const PRUnichar *title,const PRUnichar *text,PRUnichar **password,const PRUnichar *message,PRBool *value,PRBool *ret) 
{
	GtkWidget *dialog;
	GtkWidget *gtk_parent = gtk_widget_from_dom_window(parent);
	GtkWidget *table;
	GtkWidget *label;
	GtkWidget *password_entry;
	GtkWidget *check_button;
	const nsACString &msg = NS_ConvertUCS2toUTF8 (text);
	gint r;

	dialog = gtk_message_dialog_new(GTK_WINDOW(gtk_parent),GTK_DIALOG_MODAL,GTK_MESSAGE_QUESTION,GTK_BUTTONS_OK_CANCEL,"%s",PromiseFlatCString(msg).get());

	table = gtk_table_new(2,2,FALSE);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox),table,FALSE,FALSE,5);

	label = gtk_label_new("Password:");
	gtk_table_attach(GTK_TABLE(table),label,0,1,0,1,(GtkAttachOptions)(GTK_EXPAND | GTK_FILL),(GtkAttachOptions)0,0,0);

	password_entry = gtk_entry_new();
	gtk_entry_set_visibility(GTK_ENTRY(password_entry),FALSE);
	gtk_table_attach(GTK_TABLE(table),password_entry,1,2,0,1,(GtkAttachOptions)0,(GtkAttachOptions)0,0,0);
	gtk_entry_set_text_from_prunichar(GTK_ENTRY(password_entry),*password);

	gtk_widget_show_all(table);

	if (value) {
		check_button = gtk_check_button_new_with_label("");
		gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox),check_button,FALSE,FALSE,5);
		gtk_button_set_text_from_prunichar(GTK_BUTTON(check_button),message);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button),*value);
		gtk_widget_show(check_button);
	}

	gtk_window_set_title_from_prunichar(GTK_WINDOW(dialog),title);

	r = gtk_dialog_run (GTK_DIALOG (dialog));
	*ret = (r == GTK_RESPONSE_OK) ? PR_TRUE : PR_FALSE;

	*password = gtk_entry_get_text_as_prunichar(GTK_ENTRY(password_entry));
	if (value) {
		*value = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button));
	}

	gtk_widget_destroy (dialog);
	
	return NS_OK;
}


NS_IMETHODIMP IshzillaPromptService::Select(nsIDOMWindow *parent,const PRUnichar *title,const PRUnichar *text,PRUint32 count,const PRUnichar **list,PRInt32 *selection,PRBool *ret)
{
	GtkWidget *dialog;
	GtkWidget *gtk_parent = gtk_widget_from_dom_window(parent);
	GtkListStore *list_store;
	GtkWidget *tree_view;
	GtkTreeSelection *tree_selection;
	GtkCellRenderer *cell_renderer;
	GtkTreeIter iter;
	const nsACString &msg = NS_ConvertUCS2toUTF8 (text);
	gint r;
	PRUint32 i;

	dialog = gtk_message_dialog_new(GTK_WINDOW(gtk_parent),GTK_DIALOG_MODAL,GTK_MESSAGE_QUESTION,GTK_BUTTONS_OK_CANCEL,"%s",PromiseFlatCString(msg).get());


	list_store = gtk_list_store_new(2,G_TYPE_STRING,G_TYPE_INT);

	tree_view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(list_store));
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox),tree_view,TRUE,TRUE,0);
	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(tree_view),FALSE);

	cell_renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(tree_view),0,"",cell_renderer,"text",0,NULL);

	for (i = 0; i < count; i++) {
		gchar *item = ToNewCString(NS_ConvertUCS2toUTF8(list[i]));

		gtk_list_store_append(list_store,&iter);
		gtk_list_store_set(list_store,&iter,0,item,1,i,-1);

		nsMemory::Free(item);
	}

	tree_selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree_view));
	gtk_tree_model_get_iter_first(GTK_TREE_MODEL(list_store),&iter);
	gtk_tree_selection_select_iter(tree_selection,&iter);

	gtk_widget_show(tree_view);
	r = gtk_dialog_run(GTK_DIALOG(dialog));

	if (gtk_tree_selection_get_selected(tree_selection,NULL,&iter)) {
		gtk_tree_model_get(GTK_TREE_MODEL(list_store),&iter,1,selection,-1);
		*ret = (r == GTK_RESPONSE_OK) ? PR_TRUE : PR_FALSE;
	} else {
		*ret = PR_FALSE;
	}

	gtk_widget_destroy(dialog);

	return NS_OK;
}













static GtkWidget * gtk_widget_from_dom_window(nsIDOMWindow *window)
{
	nsresult rv;

	nsCOMPtr<nsIWindowWatcher> watcher(do_GetService("@mozilla.org/embedcomp/window-watcher;1"));
	if (!watcher) {
		return NULL;
	}

	nsCOMPtr<nsIDOMWindow> dom_window(window);
	if (!dom_window)
	{
		rv = watcher->GetActiveWindow(getter_AddRefs(dom_window));
		if (NS_FAILED(rv) || !dom_window) {
			return NULL;
		}
	}

        nsCOMPtr<nsIWebBrowserChrome> window_chrome;
	rv = watcher->GetChromeForWindow(dom_window,getter_AddRefs(window_chrome));
	if (NS_FAILED(rv)) {
		return NULL;
	}

	nsCOMPtr<nsIEmbeddingSiteWindow> es_window(do_QueryInterface(window_chrome,&rv));
	if (NS_FAILED(rv)) {
		return NULL;
	}

        GtkWidget *moz;
	rv = es_window->GetSiteWindow((void **)&moz);
	if (NS_FAILED(rv)) {
		return NULL;
	}

	return gtk_widget_get_toplevel(moz);
}

static void gtk_window_set_title_from_prunichar(GtkWindow *window,const PRUnichar *title)
{
	const nsACString &t = NS_ConvertUCS2toUTF8(title);

	gtk_window_set_title(window,(title == NULL ? "Ishzilla" : PromiseFlatCString(t).get()));

	return;
}

static void gtk_button_set_text_from_prunichar(GtkButton *button,const PRUnichar *text)
{
	const nsACString &t = NS_ConvertUCS2toUTF8 (text);
	
	gtk_button_set_label(button,PromiseFlatCString(t).get());
	
	return;
}

static void gtk_label_set_text_from_prunichar(GtkLabel *label,const PRUnichar *text)
{
	const nsACString &t = NS_ConvertUCS2toUTF8 (text);
	
	gtk_label_set_text(label,PromiseFlatCString(t).get());
	
	return;
}

static void gtk_entry_set_text_from_prunichar(GtkEntry *entry,const PRUnichar *text)
{
	const nsACString &t = NS_ConvertUCS2toUTF8 (text);
	
	gtk_entry_set_text(entry,PromiseFlatCString(t).get());
	
	return;
}

static PRUnichar * gtk_entry_get_text_as_prunichar(GtkEntry *entry)
{
	const gchar *t = gtk_entry_get_text(entry);

	return ToNewUnicode(NS_ConvertUTF8toUCS2(t));
}
